//
//  PTController.h
//  PTSDK
//
//  Copyright 2013 PunchTab. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>

#if !__has_feature(objc_arc)
#error PTSDK must be built with ARC.
// You can turn on ARC for only PTSDK files by adding -fobjc-arc to the build phase for each of its files.
#endif

/**
 Response block for all network calls. If successful, success will be true
 and the response will be populated with the custom object for the api.
 If it fails, success will be false and the response will be an NSError object.
 */
typedef void (^ResponseBlock)(BOOL success, id response);

@interface PTController : NSObject {
    
}

@property (nonatomic, strong) NSDictionary * userData;
@property (unsafe_unretained, nonatomic, readonly) NSString * ptToken;
@property (nonatomic, strong) NSString * clientId;

@property (nonatomic, assign) BOOL disableOverlay;

/**
 @returns whether or not there is already a valid token stored
 */
@property (readonly) BOOL hasToken;

/**
 Pings the server so that a user that has downloaded the app through a link from a Giveaway will receive entries in the Giveaway. An App ID is not required for this method.
 */
+ (void)enterGiveaway;

/**
 Handler for the single sign-on processing
 @param url - the url of the redirect
 @returns whether or not the method handled the url
 */
- (BOOL)handleOpenURL:(NSURL *)url;

/**
 Constructor to use punchtab for authentication
 @param clientId - identifier received from punchtab.com
 @returns instance of PTController
 */
- (id)initWithClientId:(NSString *)clientId;

/**
 Constructor to use SSO for authentication
 @param clientId - identifier received from punchtab.com
 @param domainName - domain of your website
 @param accessKey - access key received from punchtab.com
 @param secretKey - secret key received from punchtab.com
 @returns instance of PTController
 */
- (id)initWithClientId:(NSString *)clientId domainName:(NSString *)domainName accessKey:(NSString *)accessKey secretKey:(NSString *)secretKey;

/**
 Logs the user in. If there is an existing token, it is used and the response is immediately returned. If not the user is sent through the external sign-on process.
 Success Response - NSDictionary containing raw JSON user data
 */
- (void)loginBlock:(ResponseBlock)block;

/**
 Logs the user in. If there is an existing token, it is used and the response is immediately returned. If not the user is sent through the single sign-on process.
 @param userId - unique identifier of the user
 @param firstName - first name of the user
 @param lastName - last name of the user
 Success Response - NSDictionary containing raw JSON user data
 */
- (void)loginWithUserId:(NSString *)userId firstName:(NSString *)firstName lastName:(NSString *)lastName email:(NSString *)email block:(ResponseBlock)block;

/**
 Logs the user in. If there is an existing token, it is used and the response is immediately returned. If not the user is sent through the single sign-on process.
 @param userId - unique identifier of the user
 @param firstName - first name of the user
 @param lastName - last name of the user
 @param email - email address of the user
 Success Response - NSDictionary containing raw JSON user data
 */
- (void)loginWithUserId:(NSString *)userId firstName:(NSString *)firstName lastName:(NSString *)lastName block:(ResponseBlock)block;

/**
 Logs the user out. Clears out all local PunchTab related data.
 Success Response - server response if present
 */
- (void)logoutBlock:(ResponseBlock)block;

/**
 Gets the leaderboard for the reward program.
 Success Response - NSArray of users in the leaderboard
 */
- (void)getLeaderboardBlock:(ResponseBlock)block;

/**
 Gets the leaderboard for the reward program.
 @param showUser - whether or not the user should see their relative placement in the leaderboard
 Success Response - NSArray of users in the leaderboard
 */
- (void)getLeaderboardWithMe:(BOOL)showUser block:(ResponseBlock)block;

/**
 Gets the user's PunchTab related activity stream
 Success Response - NSArray of user's activities
 */
- (void)getActivitiesBlock:(ResponseBlock)block;

/**
 Gets the user's PunchTab related activity stream
 @param filter - filter for a specific activity type
 Success Response - NSArray of user's activities
 */
- (void)getActivitiesFilter:(NSString *)filter block:(ResponseBlock)block;


/**
 Gets the user's PunchTab related activity stream
 @param limit - the max activities returned
 Success Response - NSArray of user's activities
 */
- (void)getActivitiesLimit:(int)limit block:(ResponseBlock)block;

/**
 Gets the user's PunchTab related activity stream
 @param filter - filter for a specific activity type
 @param limit - the max activities returned
 Success Response - NSArray of user's activities
 */
- (void)getActivitiesFilter:(NSString *)filter limit:(int)limit block:(ResponseBlock)block;

/**
 Gets the list of rewards available to the user
 Success Response - NSArray of rewards for the program
 */
- (void)getRewardsBlock:(ResponseBlock)block;

/**
 Gets the list of rewards available to the user
 @param limit - the max rewards returned
 Success Response - NSArray of rewards for the program
 */
- (void)getRewardsLimit:(int)limit block:(ResponseBlock)block;

/**
 Send an activity to the server to score points. Default is 100 points.
 Success Response - NSDictionary server response
 */
- (void)sendActivity:(NSString *)activity block:(ResponseBlock)block;

/**
 Send an activity to the server to score points. Default is 100 points.
 @param points - the number of points to score
 Success Response - NSDictionary server response
 */
- (void)sendActivity:(NSString *)activity withPoints:(int)points block:(ResponseBlock)block;

- (void)sendActivity:(NSString *)activity withPoints:(int)points refId:(NSNumber *)refId block:(ResponseBlock)block;

/**
 Send an activity to the server. This is more of a direct interface to the API.
 @param parameters - custom parameters sent to the server
 Success Response - NSDictionary server response
 */
- (void)sendActivity:(NSString *)activity parameters:(NSDictionary *)parameters block:(ResponseBlock)block;

/**
 Forces a call to the server to update the local userData. Use the userData property in order to access local userData.
 */
- (void)getUserData;

/**
 Send a the redeem activity to the server with a reward id.
 @param rewardId - the id to redeem from the getRewards method
 Success Response - NSDictionary server response
 */
- (void)redeemReward:(NSNumber *)rewardId block:(ResponseBlock)block;

@end

